@@ -1,10 +1,51 @@
+#!/usr/bin/perl -w
 package Religion::Bible::Regex::Builder;
 
-use warnings;
-use strict;
-use Carp;
+=head1 NAME
 
-use version; our $VERSION = qv('0.9.2');
+Religion::Bible::Regex::Builder - builds regular expressions that match Bible References 
+
+=head1 VERSION
+
+version 0.9
+
+=cut
+
+our $VERSION = '0.9';
+
+=head1 SYNOPSIS
+
+use warnings;
+
+use Religion::Bible::Regex::Config;
+use Religion::Bible::Regex::Builder;
+
+my $configfile = 'config.yml';
+
+my $c = new Religion::Bible::Regex::Config($configfile);
+my $r = new Religion::Bible::Regex::Builder($c);                                                                                                                     
+my $text = "Ge 1:1, Mt 6:33, see page 4:5 and Jn 3:16";
+$text =~ s/$r->{reference_biblique}/<ref id="$&">$&<\/ref>/g;
+
+print $text . "\n";
+
+--------
+This prints:
+<ref id="Ge 1:1">Ge 1:1</ref>, <ref id="Mt 6:33">Mt 6:33</ref>, see page 4:5 and <ref id="Jn 3:16">Jn 3:16</ref>
+
+=head1 DESCRIPTION
+This module builds highly configurable regular expressions for parsing Bible references.
+The goal of this project is to make higher level Bible viewing, editing and tagging tools easier to create.
+The configuration files are in YAML format.
+
+Also included with this project are configurations files for Bible verses in different languages.
+=cut
+
+
+use strict;
+use warnings;
+#use Cwd;
+use Carp;
 use Data::Dumper;
 # Input files are assumed to be in the UTF-8 strict character encoding.
 use utf8;
@@ -14,12 +55,31 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
 
 require Exporter;
 our @ISA = qw(Exporter);
+
 our %EXPORT_TAGS = ( 'all' => [ qw( ) ] );
+
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
+
 our @EXPORT = qw(
 
 );
 
+=head2 METHODS
+
+=over 4
+
+=item new($$;$$)
+
+Builds the set of regular expressions for parsing Bible references.
+
+Parameters:
+    1. A Religion::Bible::Regex::Config object which gives configurations
+       such as the Books and Abbreviations to recognize, key phrases which 
+       mark the beginning of a verse or list of verses, etc ... For more, 
+       information see the 
+
+=cut
+
 sub new {
     my $class = shift;
     my $config = shift;
@@ -27,17 +87,16 @@ sub new {
     bless $self, $class;
 
     # Get the Configurations for building these regular expressions
-    my %configs;
-    $self->_process_config($config->get_search_configurations, \%configs);
+    my %bookconfig = $self->process_config($config);
 
-    ######################################################################################
+    #################################################################################### 
     #	Définitions par défaut des expressions régulières avec références bibliques
     #  
-    # 	La fonction '_set_regex' a trois paramètres.
+    # 	La fonction 'set_regex' a trois paramètres.
     #		1. Un nom unique pour cette expression régulière
     #		2. Une experssion régulière
     #		3. Si la paramètre deux est 'undef', une experssion régulière comme defaut 
-    ###################################################################################### 
+    #################################################################################### 
     
     my $spaces = qr/([\s ]*)/;
 
@@ -46,23 +105,23 @@ sub new {
     #################################################################################### 
     # chapitre : c'est un nombre qui indique un chapitre
     my $chapitre = qr/\d{1,3}/;
-    $self->_set_regex(	'chapitre', 
-			$configs{'chapitre'}, 
+    $self->set_regex(	'chapitre', 
+			$bookconfig{'chapitre'}, 
             $chapitre
 		    );
 
     # verset : c'est un nombre qui indique un verset
     my $verset = qr/\d{1,3}[abcdes]?/;
-    $self->_set_regex(	'verset', 
-			$configs{'verset'}, 
-			$verset
+    $self->set_regex(	'verset', 
+			$bookconfig{'verset'}, 
+            $verset
 		    );
 
     # chiffre : c'est un nombre qui indique un chapitre ou verset 
     my $chiffre = qr/\d{1,3}[abcdes]?/;
-    $self->_set_regex(	'chiffre', 
-			$configs{'chiffre'}, 
-			$chiffre
+    $self->set_regex(	'chiffre', 
+			$bookconfig{'chiffre'}, 
+            $chiffre
 		    );
 
 
@@ -71,45 +130,44 @@ sub new {
     #################################################################################### 
     # cv_separateur : vous pouvez trouver ce entre un chapitre et un verset
     my $cv_separateur = qr/(?::|\.)/;
-    $self->_set_regex(	'cv_separateur', 
-			$configs{'cv_separateur'}, 
-			$cv_separateur	
+    $self->set_regex(	'cv_separateur', 
+			$bookconfig{'cv_separateur'}, 
+		    $cv_separateur	
 		    );
 
     # separateur : cette sépare deux références bibliques
     my $separateur = qr/\bet\b/;
-    $self->_set_regex(	'separateur', 
-			$configs{'separateur'}, 
-			$separateur	
+    $self->set_regex(	'separateur', 
+			$bookconfig{'separateur'}, 
+    		$separateur	
 		    );
 
     # cl_separateur : cette sépare deux références bibliques et que le deuxième référence est un référence d'un chaptire
     my $cl_separateur = qr/;/;
-    $self->_set_regex(	'cl_separateur', 
-			$configs{'cl_separateur'}, 
-			$cl_separateur	
+    $self->set_regex(	'cl_separateur', 
+			$bookconfig{'cl_separateur'}, 
+    		$cl_separateur	
 		    );
 
     # vl_separateur : cette sépare deux références bibliques et que le deuxième référence est un référence d'un verset
     my $vl_separateur = qr/,/;
-    $self->_set_regex(	'vl_separateur', 
-			$configs{'vl_separateur'}, 
-			$vl_separateur		
+    $self->set_regex(	'vl_separateur', 
+			$bookconfig{'vl_separateur'}, 
+	        $vl_separateur		
 		    );
 
     my $intervale = qr/(?:-|–|−)/;
     # tiret : ce correspond à tous les types de tiret
-    $self->_set_regex(	'intervale', 
-			$configs{'intervale'}, 
-			$intervale	
+    $self->set_regex(	'intervale', 
+			$bookconfig{'intervale'}, 
+		    $intervale	
 		    );
-
     # reference_separateurs : ce correspond à tous les types de separateur entre références biblque 
     my $cl_ou_vl_separateur = qr/(?:$self->{cl_separateur}|$self->{vl_separateur}|$self->{separateur})/;
-    $self->_set_regex(	'cl_ou_vl_separateurs', 
-			$configs{'cl_ou_vl_separateurs'}, 
-			$cl_ou_vl_separateur	
-                     );
+    $self->set_regex(	'cl_ou_vl_separateurs', 
+			$bookconfig{'cl_ou_vl_separateurs'}, 
+	        $cl_ou_vl_separateur	
+		    );
 
 
     #################################################################################### 
@@ -126,9 +184,9 @@ sub new {
 
     # intervale_chiffre : deux chapitre avec un tiret entre
     # Par exemple: '-2', '–9', ou ' - 4'
-    $self->_set_regex(	'intervale_chiffre', 
-			$configs{'intervale_chiffre'},  
-                        $intervale_chiffre	
+    $self->set_regex(	'intervale_chiffre', 
+			$bookconfig{'intervale_chiffre'},  
+		    $intervale_chiffre	
 		    );
 
     my $cv_separateur_chiffre = qr/
@@ -141,54 +199,53 @@ sub new {
 
     # cv_separateur_chiffre : deux chapitre avec un tiret entre
     # Par exemple: ':2', '.9', ou ' : 4'
-    $self->_set_regex(	'cv_separateur_chiffre', 
-			$configs{'cv_separateur_chiffre'}, 
-                        $cv_separateur_chiffre	
-		     );
+    $self->set_regex(	'cv_separateur_chiffre', 
+			$bookconfig{'cv_separateur_chiffre'}, 
+		    $cv_separateur_chiffre	
+		    );
 
     #################################################################################### 
     # Définitions de les references numiques 
     #################################################################################### 
 
-    ####################################################################################
+    #######################################################################################################
     # Les mots donne contexte aux référence biblique
     # Par Exemple: 
     #   chapitre_mots: 'voir la chapitre'
     #   texte: voir la chapitre 9
     # 
-    #   Avec cette texte 'voir la chapitre' comme chapitre_mots le 9 peu être indentifié 
-    #   comme un chapitre
-    #####################################################################################
+    #   Avec cette texte 'voir la chapitre' comme chapitre_mots le 9 peu être indentifié comme un chapitre
+    #######################################################################################################
    
-    # reference_contexte_mots_avant : les mots qui indique que le prochain référence
-    #                                 est un chapitre référence
+    # reference_contexte_mots_avant : les mots qui indique que le prochain référence est un chapitre référence
     my $reference_mots = qr/(?:dans|voir aussi)/;
-    $self->_set_regex(	'reference_mots', 
-			$configs{'reference_mots'}, 
-                        $reference_mots			
+    $self->set_regex(	'reference_mots', 
+			$bookconfig{'reference_mots'}, 
+            $reference_mots			
     );
 
     # chapitre_contexte_mots_avant : les mots qui indique que le prochain référence est un chapitre référence
     my $chapitre_mots = qr/(?:dans le chapitre)/;
-    $self->_set_regex(	'chapitre_mots', 
-			$configs{'chapitre_mots'}, 
-                        $chapitre_mots			
+    $self->set_regex(	'chapitre_mots', 
+			$bookconfig{'chapitre_mots'}, 
+            $chapitre_mots			
     );
 
     # verset_contexte_mots_avant : les mots qui indique que le prochain référence est un verset référence
     my $verset_mots = qr/(?:vv?\.)/;
-    $self->_set_regex(	'verset_mots', 
-			$configs{'verset_mots'},  
-                        $verset_mots
+    $self->set_regex(	'verset_mots', 
+			$bookconfig{'verset_mots'},  
+            $verset_mots
     );
 
     # voir_contexte_mots_avant : les mots qui indique que le prochain référence est un verset référence
     my $voir_mots = qr/(?:voir)/;
-    $self->_set_regex(	'voir_mots', 
-			$configs{'voir_mots'},  
-                        $voir_mots
+    $self->set_regex(	'voir_mots', 
+			$bookconfig{'voir_mots'},  
+            $voir_mots
     );
 
+
     #################################################################################### 
     # Définitions de les expressions avec livres 
     #################################################################################### 
@@ -198,17 +255,17 @@ sub new {
         Samuel|S|Rois|R|Chroniques|Ch|Corinthiens|Co|Thessaloniciens|Th|Timothée|Ti|Pierre|P|Jean|Jn|Esras|Es|Maccabees|Ma|Psalm|Ps
     /x;
 
-    $self->_set_regex(	'livres_numerique', 
-			$configs{'livres_numerique'}, 
-                        $livres_numerique
+    $self->set_regex(	'livres_numerique', 
+			$bookconfig{'livres_numerique'}, 
+            $livres_numerique
     );
 
     my $livres_numerique_protect = "";
     if (defined($self->{'livres_numerique'})) {
         $livres_numerique_protect = qr/(?!(?:[\s ]*(?:$livres_numerique)))/;
     }
-    $self->_set_regex(   'livres_numerique_protect',
-            $configs{'livres_numerique_protect'},
+    $self->set_regex(   'livres_numerique_protect',
+            $bookconfig{'livres_numerique_protect'},
             $livres_numerique_protect
     );
 
@@ -218,9 +275,9 @@ sub new {
 /x;
 
     # livres : le nom complet de tous les livres, avec et sans accents
-    $self->_set_regex(	'livres', 
-			$configs{'livres'}, 
-                        $livres
+    $self->set_regex(	'livres', 
+			$bookconfig{'livres'}, 
+            $livres
     );
 
     my $abbreviations = qr/
@@ -228,17 +285,17 @@ sub new {
 /x;
     
     # abbreviations : le nom complet de tous les abbreviations, avec et sans accents
-    $self->_set_regex(	'abbreviations', 
-			$configs{'abbreviations'}, 
-                        $abbreviations
-	);
+    $self->set_regex(	'abbreviations', 
+			$bookconfig{'abbreviations'}, 
+            $abbreviations
+    );
 
     # livres_et_abbreviations : la liste de tous les livres et les abréviations
     my $livres_et_abbreviations = qr/(?:$self->{'livres'}|$self->{'abbreviations'})/;
-    $self->_set_regex(	'livres_et_abbreviations', 
-			$configs{'livres_et_abbreviations'}, 
-                        $livres_et_abbreviations
-	);
+    $self->set_regex(	'livres_et_abbreviations', 
+			$bookconfig{'livres_et_abbreviations'}, 
+            $livres_et_abbreviations
+		    );
 
     # contexte_mots : Tous les mots qui viennent avant une référence biblique. Des mots différents peut 
     #                fournir des contextes différents. Par exemple, 'voir le chapitre' fournit une 
@@ -255,46 +312,56 @@ sub new {
       )
     /x;
 
-    $self->_set_regex(	'contexte_mots', 
-			$configs{'contexte_mots'}, 
-			$contexte_mots
+    $self->set_regex(	'contexte_mots', 
+			$bookconfig{'contexte_mots'}, 
+            $contexte_mots
 		    );
 
-    #livre2abre : une table de changement pour livre à l'abréviation
-    $self->_set_hash(	'book2key', 
-			$configs{'book2key'}, 
-                        {}
+    #livre2abre : une table de changement du livre à l'abréviation
+    $self->set_hash(	'book2key', 
+			$bookconfig{'book2key'}, 
+            {}
     );
     
-    #abre2livres : une table de changement pour abréviation à livre
-    $self->_set_hash(	'abbr2key', 
-			$configs{'abbr2key'}, 
+    #abre2livres : une table de changement du abréviation à livre
+    $self->set_hash(	'abbr2key', 
+			$bookconfig{'abbr2key'}, 
 			{}
 		    );
 
-    #livre2abre : une table de changement pour livre à l'abréviation
-    $self->_set_hash(	'key2book', 
-			$configs{'key2book'}, 
-                        {}
+    #livre2abre : une table de changement du livre à l'abréviation
+    $self->set_hash(	'key2book', 
+			$bookconfig{'key2book'}, 
+            {}
     );
     
-    #abre2livres : une table de changement pour abréviation à livre
-    $self->_set_hash(	'key2abbr', 
-			$configs{'key2abbr'}, 
+    #abre2livres : une table de changement du abréviation à livre
+    $self->set_hash(	'key2abbr', 
+			$bookconfig{'key2abbr'}, 
 			{}
 		    );
 
 
     # livres_avec_un_chapitre :  la liste de tous les livres avec un seul chapitre
-    my $livres_avec_un_chapitre = qr/(?:Ab|Abdias|2Jn|2Jean|Phm|Philemon|Philémon|Jud|Jude|3Jn|3Jean)/;
-    $self->_set_regex(	'livres_avec_un_chapitre', 
-			$configs{'livres_avec_un_chapitre'}, 
-                        $livres_avec_un_chapitre
-	             );
+    my $livres_avec_un_chapitre = qr/Ab|Abdias|2Jn|2Jean|Phm|Philemon|Philémon|Jud|Jude|3Jn|3Jean/;
+    $self->set_regex(	'livres_avec_un_chapitre', 
+			$bookconfig{'livres_avec_un_chapitre'}, 
+            $livres_avec_un_chapitre
+		    );
 
     #######################################################################################################
-    # full_reference_protection : Il s'agit d'une expression régulière complexe. Ne pas changer, 
-    # sauf si vous savez ce que vous faites.
+    # full_reference_protection : Il s'agit d'une expression régulière complexe. Ne pas changer, sauf si vous savez ce que vous faites.
+    #$self->set_regex(	'reference_protection', 
+	#		$bookconfig{'reference_protection'}, 
+	#		"(?<!(>|\"))(?<!(1|2|3))(?<!(1\\s|2\\s|3\\s))(?<!(1 |2 |3 ))(?<!(1\\n|2\\n|3\\n))"
+	#	    );
+
+    # reference_biblique : Expression Régulière Important. Cette expression régulière cherche les références bibliques
+    # par exemple : 'Jean 1.1', 'Genèse 1', 'Mt 4:2,6', 
+    #$self->set_regex(	'reference_biblique', 
+	#		$bookconfig{'reference_biblique'}, 
+
+	#	    );
 
     my $cv_list = qr/
         $self->{'chiffre'} # LC, '22'
@@ -325,10 +392,10 @@ sub new {
     /x; 
 
     # cv_list : Combines LC, LCC, LCCV, LCCVV and LCV, LCVV, LCVCV
-    $self->_set_regex(	'cv_list', 
-			$configs{'cv_list'},
-	                $cv_list	
-		     );
+    $self->set_regex(	'cv_list', 
+			$bookconfig{'cv_list'},
+	        $cv_list	
+		    );
 
 
     # reference_biblique_list : Cette expression régulière correspond à une liste de références bibliques 
@@ -352,10 +419,10 @@ sub new {
     )
     /x;
 
-    $self->_set_regex(	'reference_biblique', 
-			$configs{'reference_biblique'}, 
-                        $reference_biblique
-		     );
+    $self->set_regex(	'reference_biblique', 
+			$bookconfig{'reference_biblique'}, 
+		    $reference_biblique
+		    );
 
     # reference_biblique_list : Cette expression régulière correspond à une liste de références bibliques 
     #				ex. '1 Ti 1.19 ; Ge 1:1, 2:16-18' or '1 Ti 1.19 ; 2Ti 2:16-18'
@@ -379,8 +446,8 @@ sub new {
     )
     /x;
 
-    $self->_set_regex(	'reference_biblique_list', 
-			$configs{'reference_biblique_list'}, 
+    $self->set_regex(	'reference_biblique_list', 
+			$bookconfig{'reference_biblique_list'}, 
 		    $reference_biblique_list
 		    );
 
@@ -389,13 +456,19 @@ sub new {
 
 
 ################################################################################
-# Helper functions for internal use
+# Helper functions
 ################################################################################
-sub _set_regex {
+=over 5
+
+=item set_regex 
+
+For internal use only.  Please, do not call this function
+
+=cut
+sub set_regex {
     my ($self, $key, $regex, $default_regex) = @_;
-#    return '' if (m/^$/ =~ $regex);
 	if (defined($regex)) {
-        my $result = qr/$regex/;	        # Evaluate that line
+        my $result = qr/$regex/;	# Evaluate that line
         if ($@) {                       	# Check for compile or run-time errors.
             croak "Invalid regex:\n $regex";
         } else {
@@ -408,44 +481,44 @@ sub _set_regex {
 	}
 }
 
-sub _set_hash {
+sub set_hash {
     my ($self, $key, $hash, $default_hash) = @_;
     if (defined($hash)) {
             $self->{$key} = $hash;
     } else {
         $self->{$key} = $default_hash;
     }
+
 }
 
 ################################################################################
 # les fonctions qui se préoccupe de la configuration
 ################################################################################
-sub _process_config {
+sub process_config {
     my $self = shift;
     my $config = shift;
-    my $retval = shift;
+    my %retval;
 
-    # If this is the book configurations then build the associated data structures
-    # If this configuration value is a file name, then use the contents of that
-    #    file to build a regular expression
-    # If the configuration value is a HASH, then recursively call  _process_config
-    # If this configuration value is a string, then copy it to the data structure
-    #    that is being returned
-    while ( my ($key, $value) = each(%{$config}) ) {
-        if ($key =~ m/books/) {
-            $self->_init_book_and_abbreviation_data_structures($value, $retval);
+    while ( my ($key, $value) = each(%{$config->{mainconfig}{regex}}) ) {
+        if ($key =~ m/definitions/) {
+            $self->init_data_structures($value, $config, \%retval);
         } elsif ($value =~ m/^(?:fichier|file):/) {
-            $retval->{$key} = $self->_build_regexes_from_file($value);
+            $retval{$key} = $self->build_regexes_from_file($value);
         } elsif (defined(ref $value) && ref $value eq "HASH") {
-            $self->_process_config($value, $retval);
+            $retval{$key} = $self->process_config($value);
         } else {
-            $retval->{$key} = $value;    
+            $retval{$key} = $value;    
         }
     }
-    return $retval;
+    return %retval;
 }
 
-sub _build_regexes_from_file {
+# sub data {
+#     my $self = shift;
+#     return $self->{'data'};
+# }
+
+sub build_regexes_from_file {
     my $self = shift;
     my $value = shift;
     my @list;
@@ -453,7 +526,7 @@ sub _build_regexes_from_file {
     # Enleve le phrase 'fichier:' ou 'file:'
     $value =~ s/^(?:fichier|file)://g;
     
-    open(*LIST, "<:encoding(UTF-8)", $value) or croak "Couldn't open \'$value\' for reading: $!\n";
+    open(LIST, "<:encoding(UTF-8)", $value) or croak "Couldn't open \'$value\' for reading: $!\n";
     while(<LIST>) {
         chomp;                  # no newline
         s/[^\\]#.*//;           # no comments si il y a un '\' devant le '#' il n'est pas un commentarie
@@ -463,21 +536,20 @@ sub _build_regexes_from_file {
         push @list, $_;
     }
     close (LIST);
-    return "(?:" . _join_regex(\@list) . ")";
+    return "(?:" . join_regex(\@list) . ")";
 }
 
-sub _join_regex {
+sub join_regex {
 	my $array_ref = shift;
 	if (defined($array_ref)) {
 		return join("|", @{$array_ref});
 	} else { 
-		return;
+		return undef;
 	}
 }
 
-
 ################################################################################
-# _init_book_and_abbreviation_data_structures
+# init_data_structures
 # 
 # Creates the following mappings:
 #   An array of all match book names (book names to search for in a document)
@@ -497,17 +569,19 @@ sub _join_regex {
 #        Abbreviation: Ge
 #
 ################################################################################
-sub _init_book_and_abbreviation_data_structures {
+sub init_data_structures {
     my $self = shift;
+    my $path_to_config_file = shift;
     my $config = shift;
     my $retval = shift;
 
+    my $bookconfig = $config->{bookconfig};
     my $regex;
     my (@livres, @livres_numerique, @abbreviations);    # Array for all match books and another for match books starting with a number
     my (%book2key, %abbr2key, %key2abbr, %key2book);    # Mappings between match books and abbreviations and the primary key
     
     # Loop through each number and gather the books
-    while ( my ($key, $value) = each %{$config} ) {
+    while( my ($key, $value) = each %{$bookconfig} ) {
         # Loop through 
         foreach my $livre (@{$value->{Match}{Book}}) {
             push @livres, $livre;
@@ -523,9 +597,9 @@ sub _init_book_and_abbreviation_data_structures {
         $key2book{$key} = $value->{Normalized}{Book};
     }
      
-    $retval->{'livres'} = _join_regex(\@livres);
-    $retval->{'abbreviations'} = _join_regex(\@abbreviations);
-    $retval->{'livres_numerique'} = _join_regex(\@livres_numerique);
+    $retval->{'livres'} = join_regex(\@livres);
+    $retval->{'abbreviations'} = join_regex(\@abbreviations);
+    $retval->{'livres_numerique'} = join_regex(\@livres_numerique);
 
     $retval->{'livres_array'} = \@livres;
     $retval->{'abbreviations_array'} = \@abbreviations;
@@ -535,86 +609,126 @@ sub _init_book_and_abbreviation_data_structures {
     $retval->{'abbr2key'}   = \%abbr2key;
     $retval->{'key2book'}   = \%key2book;
     $retval->{'key2abbr'}   = \%key2abbr;
-    $retval->{'configs'} = $config;
+    $retval->{'bookconfig'} = $bookconfig;
 }
 
+=item abbreviation($key)
 
-1; # Magic true value required at end of module
-__END__
-
-=head1 NAME
-
-Religion::Bible::Regex::Builder - builds regular expressions that match Bible References
-
-
-=head1 VERSION
-
-This document describes Religion::Bible::Regex::Builder version 0.9.1
-
-
-=head1 SYNOPSIS
-
-	use Religion::Bible::Regex::Builder;
-
-	use warnings;
-
-	use Religion::Bible::Regex::Config;
-	use Religion::Bible::Regex::Builder;
-
-	my $configfile = 'config.yml';
-
-	my $c = new Religion::Bible::Regex::Config($configfile);
-	my $r = new Religion::Bible::Regex::Builder($c);
-	my $text = "Ge 1:1, Mt 6:33, see page 4:5 and Jn 3:16";
-	$text =~ s/$r->{reference_biblique}/<ref id="$&">$&<\/ref>/g;
-
-	print $text . "\n";
-
---------
-This prints:
-<ref id="Ge 1:1">Ge 1:1</ref>, <ref id="Mt 6:33">Mt 6:33</ref>, see page 4:5 and <ref id="Jn 3:16">Jn 3:16</ref>
-
-  
-=head1 DESCRIPTION
-
-This module builds highly configurable regular expressions for parsing Bible references.
-The goal of this project is to make higher level Bible viewing, editing and tagging tools easier to create.
-The configuration files are in YAML format.
-
-
-=head1 FUNCTIONS
-
-=head2 new
-
-Builds the set of regular expressions for parsing Bible references.
+Returns a string representing the normalized abbreviation for a given 
+book or key.
 
 Parameters:
-    1. A Religion::Bible::Regex::Config object which gives configurations
-       such as the Books and Abbreviations to recognize, key phrases which 
-       mark the beginning of a verse or list of verses, etc ...
+    Key - Maybe a number or a book name
+          The key is defined in the abbreviation configuration file.
+        
+          For example, '1' probably corresponds to "Genesis"
+          and the abbreviation "Ge" probably go with "Genesis" as well.
 
-=head1 DEPENDENCIES
+=cut
 
-=for author to fill in:
-	Religion::Bible::Regex::Config
+sub abbreviation {
+    my $self = shift;
+    my $key = shift;
+    # try a lookup just in case $key eq 'Pr' or 'Genèse'    
+    my $foundkey = $self->key($key);
 
-=head1 BUGS AND LIMITATIONS
+    # if we found a key then use it as the index
+    return undef unless (non_empty($foundkey));
+    return $self->{key2abbr}{$foundkey};
+}
 
-No bugs have been reported.
+sub non_empty {
+    my $value = shift;
+    return (defined($value) && $value ne '');
+}  
 
-Please report any bugs or feature requests to
-C<bug-religion-bible-regex-builder@rt.cpan.org>, or through the web interface at
-L<http://rt.cpan.org>.
+=item book($key)
 
+Returns a string representing the normalized book name for a given 
+book or key.
+
+Parameters:
+    Key - Maybe a number or a book name
+          The key is defined in the abbreviation configuration file.
+        
+          For example, '1' probably corresponds to "Genesis"
+          and the abbreviation "Ge" probably go with "Genesis" as well.
+
+=cut
+sub book {
+    my $self = shift;
+    my $key = shift;
+
+    # try a lookup just in case $key eq 'Pr' or 'Genèse'    
+    my $foundkey = $self->key($key);
+
+    # if we found a key then use it as the index
+    return undef unless (non_empty($foundkey));
+    return $self->{key2book}{$foundkey};
+}
+
+=item key($book_or_abbreviation)
+
+Returns a string representing the key book name or abbreviations
+for a given book or key.
+
+Parameters:
+    Book_or_abbreviation: is a string defined in the abbreviation
+                          configuration file under Key.Match.Book
+                          or Key.Match.Abbreviation 
+
+=cut
+sub key {
+    my $self = shift;
+    my $book_or_abbr = shift || '';
+    return $self->{book2key}{$book_or_abbr} || $self->{abbr2key}{$book_or_abbr};
+}
+
+=item bookname_type($book)
+
+Takes a string representing a normalized book or abbreviation 
+and returns :
+    CANONICAL_NAME - if the book matches a Key.Normalized.Book value
+                     defined in the abbreviation config file.
+                    
+                     This also means that given a Builder object
+                     $book =~ m/$builder->{livres}/ is true.
+                     
+    ABBREVIATION   - if the book matches a Key.Normalized.Abbreviation
+                     value defined in the abbreviation config file.
+                    
+                     This also means that given a Builder object
+                     $book =~ m/$builder->{abbreviations}/ is true.
+
+=cut
+sub bookname_type {
+    my $self = shift;
+    my $book = shift || '';
+    return('NONE') unless non_empty($book);
+    return('CANONICAL_NAME') if ($book =~ m/$self->{livres}/);
+    return('ABBREVIATION') if ($book =~ m/$self->{abbreviations}/);
+    return('UNKNOWN');
+}
+
+sub is_canonical_name {
+    my $self = shift;
+    my $book = shift || '';
+    return ($self->bookname_type($book) eq 'CANONICAL_NAME') ? 1 : undef;
+}
+
+sub is_abbreviation {
+    my $self = shift;
+    my $abbr = shift || '';
+    return ($self->bookname_type($abbr) eq 'ABBREVIATION') ? 1 : undef;
+}
 
 =head1 AUTHOR
 
-Daniel Holmlund  C<< <holmlund.dev@gmail.com> >>
+Daniel Holmlund<< <holmlund.dev at gmail.com> >>
+
+=head1 SPONSORSHIP
+
+Development sponsored by Editions Clé <http://editionscle.com/>.
 
 
-=head1 LICENCE AND COPYRIGHT
-
-Copyright (c) 2009, Daniel Holmlund C<< <holmlund.dev@gmail.com> >>. All rights reserved.
-
-This module is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself. See L<perlartistic>.
+1;
